//
//
//
//
//
//
//

#include <pic18.h>
#include <pic18fx490.h>
#include "lcd.h"
#include "adc.h"
#include "clock.h"
#include "timers.h"
#include "usart.h"
#include "lcd_test.h"

#define	BIT0	0x01
#define	BIT1	0x02
#define	BIT2	0x04
#define	BIT3	0x08
#define	BIT4	0x10
#define	BIT5	0x20
#define	BIT6	0x40
#define	BIT7	0x80


/**** VARIABLE ****/
// variable for loop
unsigned int 	adc		= 0;
unsigned char 	ch		= 0;

//unsigned char	b1_press	= 0;
unsigned char	b2_press	= 0;
unsigned char	b3_press	= 0;

extern char 	update;


/**** FUNCTIONS ****/
// simple delay ~mS
void Delay_mS(unsigned long a) { 
	 a *= 370;
	while (--a!=0); 
}

// simple delay ~100us
void Delay_100uS(unsigned long a) { 
	 a *= 37;
	while (--a!=0); 
}


// BUZZER -------------------------------------------
// Init buzzer
void InitBuzzer(void) {
	
	// buzzer as output
	TRISC &= ~BIT2; 

	// buzzer pin to low
	PORTC &= ~BIT2; 

}

// Beep ~2Khz
void Beep(void) {

	unsigned char i;

	for(i=0; i<100; i++) {
		PORTC &= ~BIT2; 
		Delay_100uS(1);
		PORTC |= BIT2; 
		Delay_100uS(1);	
	}
}

// LED -----------------------------------------------
// Init Led
void InitLed(void) {
	
	// led as output
	TRISB &= ~BIT5; 

	// led off -> high
	PORTB |= BIT5; 
}

// Led blink with ~ one second
void LedBlink(void) {
	
	PORTB &= ~BIT5; 
	Delay_mS(1000);
	PORTB |= BIT5; 
	Delay_mS(1000);
}

// Led ON
void LedOn(void) {
	// Set pin to low
	PORTB &= ~BIT5; 
}

// LedOFF
void LedOff(void) {
	// Set pin to high
	PORTB |= BIT5;	
}


// LED -----------------------------------------------
// Init Buttons
void InitButtons(void) {
	
	// Center button is on port -> input
	TRISB |= (BIT0|BIT3|BIT4);
}



// MAIN FUNCTION
int main() {	

	// all as input
	ADCON1 	= 0x0F;
	CMCON	= 0x07;
	TRISA 	= 0xFF;
	TRISC 	= 0xFF;
	TRISB 	= 0xFF;

	// Init Usart1
	InitUsart1();

	// Led Init
	InitLed();

	// Buzzer Init 
	InitBuzzer();

	// Init Juystick
	InitButtons();	

	// Init ADC
	InitADC();

	// Init LCD
	InitLCD();

	// InitTimer1
	InitTimer1();

	// InitClock
	InitializeClock();

	// Init Usart1
	InitUsart1();

	// Buzzer Init 
	InitBuzzer();

	// All LCD  
	LCD_light_all();	
	Delay_mS(1000);
	
	// Clear LCD
	LCD_clear_all();	
	Delay_mS(1000);

	// Write to LCD
	FormatChar_1('P'); 
	FormatChar_2('I'); 
	FormatChar_3('C'); 
	FormatChar_4('-'); 
	FormatChar_5('L'); 
	FormatChar_6('C'); 	
	FormatChar_7('D');


	Usart1WriteText("\n\rWelcome to PIC-LCD development board\0");
	Usart1WriteText("\n\r      WWW.OLIMEX.COM/DEV   \0");
	Usart1WriteText("\n\r\0");
	Usart1WriteText("\n\rPlease Test:\0");
	Usart1WriteText("\n\r1. Button1 -> Beep buzzer, Turn on led.\0");
	Usart1WriteText("\n\r2. Button2 -> Measure temperature. \0");
	Usart1WriteText("\n\r3. Button3 -> Test LCD segment. \0");
	Usart1WriteText("\n\r4. Echo for uart\0");
	Usart1WriteText("\n\r5. Press T to make extension test. \0");
	Usart1WriteText("\n\r\0");	

	// Enable interrupt
	PEIE = 1;	
	ei();	

	while(1) {
		
		// if Button B1 is pressed
		if((PORTB & BIT3)==0) {
			Beep();
			LedOn();
			continue;
		}
		else {
			LedOff();
		}

		// if Button B2 is pressed
		if(((PORTB & BIT4)==0)&&(b3_press==0)) {
			adc = GetValueAN1();
			CalcTemp(adc);
			b3_press = 1;
		}
		if(((PORTB & BIT4)==BIT4)&&(b3_press==1)) {
			
			b3_press = 0;
		}

		// if Button B3 is pressed
		if(((PORTB & BIT0)==0)&&(b3_press==0)) {
			TestLCD();
			b3_press = 1;

		}
		if(((PORTB & BIT0)==BIT0)&&(b3_press==1)) {

			b3_press = 0;
		}

		
		if(update) {
			LCDDATA18 ^= 0x08;
			LCDPrintClck(GetHour(), GetMin());
			update = 0;	
		}

		ch = Usart1ReadChar_nonstop();
		if(ch != 0) {
			if(ch == 'T') { TestExt(); continue; }
			Usart1WriteChar(ch);
			Usart1WriteChar('*');
			ch = 0;
		}

	}	

	return 0;

}
